(* ------------------------------------------------------ *)
(* DCS.PAS                                                *)
(* Turbo Vision Dialog Construction Set                   *)
(* Generiert Turbo Vision Dialogbox-Quellcode             *)
(* Autor: Dieter Schmalstieg                              *)
(* Projektstart: 91-01-08                                 *)
(* Letztes Update: 91-02-13                               *)
(* ------------------------------------------------------ *)

PROGRAM DCS;

USES
  Objects, Drivers, Views, Menus, Dialogs,
  App, TVplus, StdDlg, Dos, MsgBox;

CONST
  cmDefin           = 103;
  cmLaden           = 104;
  cmSpeic           = 105;
  cmSourc           = 106;
  cmNeu             = 107;
  cmTeste           = 108;
  cmAende           = 109;
  cmLoesc           = 110;
  cmEinfu           = 111;
  cmSpeiU           = 112;
  cmNeuDi           = 113;
  cmKopie           = 114;
  cmKuA             = 115;
  cmUeber           = 116;

  hcn               = hcNoContext;
  MaxElements       = 99;

  ElTyp : ARRAY[1..8] OF STRING[11] = (
    'Button',
    'InputLine',
    'CheckBoxes',
    'RadioButton',
    'StaticText',
    'Longint',
    'Real',
    'Datum');

TYPE
  Zahl = LONGINT;

  CoordRec = RECORD
    x1, y1, x2, y2 : Zahl;
  END;

  ElementRec = RECORD
    typ : WORD;
    haslabel : WORD;
    labeltext : STRING[40];
    la_co : CoordRec;
    el_co : CoordRec;
    subelem : Zahl;
    subname : ARRAY[1..16] OF STRING[20];
  END;

CONST
  LeerElement : ElementRec =
    (typ : 0;
    haslabel : 0;
    labeltext : '';
    la_co : (x1 : 0; y1 : 0; x2 : 0; y2 : 0);
    el_co : (x1 : 0; y1 : 0; x2 : 0; y2 : 0);
    subelem : 0;
    subname : ('','','','','','','','',
               '','','','','','','','')
  );

TYPE
  ElementField = ARRAY[0..MaxElements] OF ElementRec;
  ElementFieldPtr = ^ElementField;

  DefinitionRec = RECORD
    titel : STRING[20];
    prozedur : STRING[20];
    co : CoordRec;
  END;

CONST
  LeerDefinition : DefinitionRec = (
    titel : '';
    prozedur : '';
    co : (x1 : 5; y1 : 2; x2 : 75; y2 : 20)
  );

TYPE
  TMyApp = OBJECT(TApplication)
    PROCEDURE HandleEvent(VAR Event: TEvent); VIRTUAL;
    PROCEDURE InitMenuBar; VIRTUAL;
    PROCEDURE InitStatusLine; VIRTUAL;
    PROCEDURE Definitionen;
    PROCEDURE Laden;
    PROCEDURE Speichern;
    PROCEDURE SpeichernUnter;
    PROCEDURE Source;
    FUNCTION Element(e1 : ElementRec;
      VAR e2 : ElementRec) : BOOLEAN;
    PROCEDURE TestDialog;
  END;

  PDDialog = ^TDDialog;
  TDDialog = object(TDialog)
  END;

(* ----- Globale Variablen -------------------- *)
VAR
  def : DefinitionRec;
  e : ElementFieldPtr;
  n : INTEGER;
  g, nr : BYTE;
  bool : BOOLEAN;
  enum, enum1 : LONGINT;
  filename : PathStr;
  saved : BOOLEAN;

(* ----- Prozeduren --------------------------- *)
PROCEDURE ProgInit;
BEGIN
  nr := 0;
  enum := 0;
  def := LeerDefinition;
  filename := 'NONAME.DCS';
  saved := TRUE;
END;

TYPE
  TUnsorted = OBJECT(TStringCollection)
    PROCEDURE Insert(Item : POINTER); VIRTUAL;
  END;
  PUnsorted = ^TUnsorted;

  PROCEDURE TUnsorted.Insert;
  BEGIN
    TCollection.Insert(Item);
  END;

FUNCTION GetNumber(number: LONGINT; tit: STRING) : LONGINT;

VAR
  d : PDialog;
  R : TRect;
  li : PUnsorted;
  lb : PListBox;
  sb : PScrollBar;
  i : INTEGER;
  c : WORD;
  s : STRING;
BEGIN
  IF nr = 0 THEN Exit;
  R.Assign(27,2,53,18);
  d := New(PDialog, Init(R, tit));
  li := New(PUnsorted, Init(99, 0));
  FOR i := 1 TO nr DO BEGIN
    s := e^[i].labeltext;
    IF s = '' THEN s := ElTyp[e^[i].typ+1];
    li^.Insert(NewStr(s));
  END;
  R.Assign(2,11,24,12);
  sb := New(PScrollBar, Init(R));
  d^.Insert(sb);
  R.Assign(2,1,24,11);
  lb := New(PListBox, Init(R, 1, sb));
  lb^.NewList(li);
  IF number <> 0 THEN Dec(number);
  lb^.FocusItem(number);
  d^.Insert(lb);
  R.Assign(8,13,18,15);
  d^.Insert(New(PButton, Init(R, '~O~k', cmOk, bfDefault)));
  c := Desktop^.ExecView(d);
  GetNumber := lb^.Focused+1;
  Dispose(li, Done);
  Dispose(d, Done);
END;

PROCEDURE TMyApp.Laden;
VAR
  D : PFileDialog;
  i : INTEGER;
  datei : File OF ElementRec;
  workel : ElementRec;
BEGIN
  D := New(PFileDialog, Init('*.DCS', 'Laden',
    '~N~ame', fdOpenButton, 100));
  IF ValidView(D) <> NIL THEN
  BEGIN
    IF Desktop^.ExecView(D) <> cmCancel THEN
    BEGIN
      D^.GetFileName(FileName);
      {jetzt steht in FileName der Name!}
      Assign(datei, filename);
      ReSet(datei);
      Read(datei, workel);
      def.co := workel.el_co;
      def.titel := workel.labeltext;
      def.prozedur := workel.subname[1];
      i := 1;
      WHILE NOT Eof(datei) DO BEGIN
        Read(datei, e^[i]);
        Inc(i);
      END;
      Close(datei);
      nr := i-1;
      enum := 0;
    END;
    Dispose(D, Done);
  END;
END;

PROCEDURE TMyApp.Speichern;
VAR
  D : PFileDialog;
  c : WORD;
  i : INTEGER;
  datei : File OF ElementRec;
  workel : ElementRec;
BEGIN
  D := New(PFileDialog, Init('*.DCS', 'Speichern',
    '~N~ame', fdOpenButton, 100));
  IF ValidView(D) <> NIL THEN
  BEGIN
    c := cmFileOpen;
    IF filename = 'NONAME.DCS' THEN BEGIN
      c := Desktop^.ExecView(D);
      IF c = cmFileOpen THEN D^.GetFileName(FileName);
    END;
    IF c = cmFileOpen THEN BEGIN
      {jetzt steht in FileName der Name!}
      Assign(datei, filename);
      ReWrite(datei);
      workel.el_co := def.co;
      workel.labeltext := def.titel;
      workel.subname[1] := def.prozedur;
      Write(datei, workel);
      FOR i := 1 TO nr DO BEGIN
        Write(datei, e^[i]);
      END;
      Close(datei);
      saved := TRUE;
    END;
    Dispose(D, Done);
  END;
END;

PROCEDURE TMyApp.SpeichernUnter;
VAR
  D : PFileDialog;
  i : INTEGER;
  datei : File OF ElementRec;
  workel : ElementRec;
BEGIN
  D := New(PFileDialog, Init('*.DCS', 'Speichern Unter',
    '~N~ame', fdOpenButton, 100));
  IF ValidView(D) <> NIL THEN
  BEGIN
    IF Desktop^.ExecView(D) <> cmCancel THEN
    BEGIN
      D^.GetFileName(FileName);
      {jetzt steht in FileName der Name!}
      Assign(datei, filename);
      ReWrite(datei);
      workel.el_co := def.co;
      workel.labeltext := def.titel;
      workel.subname[1] := def.prozedur;
      Write(datei, workel);
      FOR i := 1 TO nr DO BEGIN
        Write(datei, e^[i]);
      END;
      Close(datei);
      saved := TRUE;
    END;
    Dispose(D, Done);
  END;
END;

PROCEDURE TMyApp.Source;
VAR
  D : PFileDialog;
  i, j : INTEGER;
  datei : Text;
  workel : ElementRec;
  fname : PathStr;
BEGIN
  D := New(PFileDialog, Init('*.INC', 'Source schreiben',
    '~N~ame', fdOpenButton, 100));
  IF ValidView(D) <> NIL THEN
  BEGIN
    IF Desktop^.ExecView(D) <> cmCancel THEN
    BEGIN
      D^.GetFileName(fname);
      Assign(datei, fname);
      ReWrite(datei);
      WriteLn(datei, 'PROCEDURE '+def.prozedur+';');
      WriteLn(datei, 'VAR');
      WriteLn(datei, '  D : PDialog;');
      WriteLn(datei, '  R : TRect;');
      WriteLn(datei, '  c : WORD;');
      WriteLn(datei, '  p : PView;');
      WriteLn(datei, 'BEGIN');
      WriteLn(datei, '  R.Assign(',
        def.co.x1:2,',',def.co.y1:2,',',
        def.co.x2:2,',',def.co.y2:2,');');
      WriteLn(datei, '  D := New(PDialog, Init(R, ',
      #39,def.titel,#39,'));');
      WriteLn(datei, '  WITH d^ DO BEGIN');
      FOR i := 1 TO nr DO WITH e^[i] DO BEGIN
        WriteLn(datei, '    R.Assign(',
        el_co.x1:2,',',el_co.y1:2,',',
        el_co.x2:2,',',el_co.y2:2,');');
        Write(datei, '    ');
        CASE typ OF
        0 : (* TButton *)
        WriteLn(datei,
          'p := New(PButton, Init(R, ',#39, labeltext, #39,
          ', cmOK, bfDefault));'
        );
        1 : (* TInputLine *)
        WriteLn(datei,
          'p := New(PInputLine, Init(R,', subelem:2, '));'
        );
        2 : BEGIN (* TCheckBoxes *)
          WriteLn(datei, 'p := New(PCheckBoxes, Init(R,');
          FOR j := 1 TO subelem DO WriteLn(datei,'        ',
            'NewSItem(',#39,subname[j],#39,',');
          Write(datei, '        NIL');
          FOR j := 1 TO subelem DO Write(datei, ')');
          WriteLn(datei);
          WriteLn(datei, '    ));');
        END;
        3 : BEGIN (* TRadiobuttons *)
          WriteLn(datei, 'p := New(PRadioButtons, Init(R,');
          FOR j := 1 TO subelem DO WriteLn(datei,'        ',
            'NewSItem(',#39,subname[j],#39,',');
          Write(datei, '        NIL');
          FOR j := 1 TO subelem DO Write(datei, ')');
          WriteLn(datei);
          WriteLn(datei, '    ));');
        END;
        4 : (* TStaticText *)
        WriteLn(datei, 'p := New(PStaticText, Init(R, ',
          #39,labeltext,#39,' ));'
        );
        5 : (* TLongint *)
        WriteLn(datei,
          'p := New(PLongint, Init(R,', subelem:2, '));'
        );
        6 : (* TReal *)
        WriteLn(datei,
          'p := New(PReal, Init(R,', subelem:2, '));'
        );
        7 : (* TDate *)
        WriteLn(datei,
          'p := New(PDate, Init(R));');
        END; (* case *)
        WriteLn(datei, '    Insert(p);');
        IF haslabel = 1 THEN BEGIN
          WriteLn(datei, '    R.Assign(',
          la_co.x1:2,',',la_co.y1:2,',',
          la_co.x2:2,',',la_co.y2:2,');');
          WriteLn(datei,
            '    Insert(New(PLabel, Init(R, ',#39,labeltext,
            #39, ' , p)));');
        END;
      END; (* for *)
      WriteLn(datei, '  END;');
      WriteLn(datei, '  c := DeskTop^.ExecView(D);');
      WriteLn(datei, '  Dispose(D, Done);');
      WriteLn(datei, 'END;');
      Close(datei);
    END;
    Dispose(D, Done);
  END;
END;

PROCEDURE TMyApp.TestDialog;
VAR
  Di : PDDialog;
  R : TRect;
  i, j : INTEGER;
  p : PView;
  c : WORD;
  strp : PSItem;
BEGIN
  R.Assign(def.co.X1, def.co.Y1, def.co.X2, def.co.Y2);
  Di := New(PDDialog, Init(R, def.titel));
  FOR i := 1 TO nr DO WITH e^[i] DO BEGIN
    R.Assign(el_co.X1, el_co.Y1, el_co.X2, el_co.Y2);
    CASE typ OF
      0 : (* TButton *)
        p := New(PButton, Init(R, labeltext,cmOK,bfNormal));

      1 : (* TInputLine *)
        p := New(PInputLine, Init(R, subelem));

      2 : (* TCheckBoxes *)
      BEGIN
        IF NOT(subelem IN [1..16]) THEN subelem := 1;
        strp := NewSItem(subname[subelem], NIL);
        FOR j := subelem-1 DOWNTO 1 DO BEGIN
          strp := NewSItem(subname[j], strp);
        END;
        p := New(PCheckBoxes, Init(R, strp));
      END;

      3 : (* TRadiobutton *)
      BEGIN
        IF NOT(subelem IN [1..16]) THEN subelem := 1;
        strp := NewSItem(subname[subelem], NIL);
        FOR j := subelem-1 DOWNTO 1 DO
          strp := NewSItem(subname[j], strp);
        p := New(PRadioButtons, Init(R, strp));
      END;

      4 : (* TStaticText *)
        p := New(PStaticText, Init(R, labeltext));

      5 : (* TLongint *)
        p := New(PLongint, Init(R, subelem));

      6 : (* TReal *)
        p := New(PReal, Init(R, subelem));

      7 : (* TDate *)
        p := New(PDate, Init(R));
    END;
    Di^.Insert(p);
    IF haslabel = 1 THEN BEGIN
        R.Assign(la_co.X1, la_co.Y1, la_co.X2, la_co.Y2);
        Di^.Insert(New(PLabel, Init(R, labeltext, p)));
    END;
  END;
  c := DeskTop^.ExecView(Di);
END;

PROCEDURE TMyApp.HandleEvent(VAR Event: TEvent);
VAR
  workel : ElementRec;
  i : INTEGER;
BEGIN
  IF (Event.What = evCommand) AND (Event.Command = cmQuit)
  THEN BEGIN
    IF NOT saved THEN
      CASE MessageBox(
        #3'Sie haben den Dialog nicht'#13+
        #3'gesichert. Speichern?',
        NIL, mfWarning+mfYesNoCancel)
      OF
        cmYes    : BEGIN Speichern; EndModal(cmQuit) END;
        cmNo     : EndModal(cmQuit);
        cmCancel : ClearEvent(event);
      END (* case *)
    ELSE EndModal(cmQuit);
  END;
  TApplication.HandleEvent(Event);
  IF Event.What = evCommand THEN BEGIN
    IF NOT(Event.Command IN [cmSpeic, cmSpeiU, cmQuit])
      THEN saved := FALSE;
    CASE Event.Command OF
      cmAende : BEGIN
        enum := GetNumber(enum, 'Element ndern');
        IF enum IN [1..nr] THEN bool :=
          Element(e^[enum], e^[enum]);
      END;
      cmLoesc : BEGIN
        enum := GetNumber(enum, 'Element lschen');
        IF enum IN [1..nr] THEN BEGIN
          FOR i := enum TO nr-1 DO e^[i] := e^[i+1];
          Dec(nr);
          IF nr < enum THEN Dec(enum);
        END;
      END;
      cmEinfu : BEGIN
        enum := GetNumber(enum, 'Element einfgen nach');
        IF enum IN [1..nr] THEN
          IF Element(LeerElement, workel) THEN BEGIN
            FOR i := nr+1 DOWNTO enum+1 DO e^[i] := e^[i-1];
            e^[enum] := workel;
            Inc(nr);
          END;
      END;
      cmKopie : BEGIN
        enum1 := GetNumber(enum, 'Kopieren von');
        IF enum1 IN [1..nr] THEN BEGIN
          enum := GetNumber(enum, 'Kopieren nach');
          IF enum IN [1..nr] THEN BEGIN
            e^[enum] := e^[enum1];
            bool := Element(e^[enum], e^[enum]);
          END;
        END;
      END;
      cmKuA : BEGIN
        enum := GetNumber(enum, 'Kopieren von');
        IF enum IN [1..nr] THEN BEGIN
          Inc(nr);
          e^[nr] := e^[enum];
          bool := Element(e^[nr], e^[nr]);
        END;
      END;
      cmNeu : IF Element(LeerElement,e^[nr+1]) THEN Inc(nr);
      cmDefin : Definitionen;
      cmTeste : TestDialog;
      cmSpeic : Speichern;
      cmLaden : Laden;
      cmSpeiU : SpeichernUnter;
      cmSourc : Source;
      cmNeuDi : ProgInit;
      cmUeber : IF MessageBox(
        #3'DCS'+#13+
        #3'Dialog Construction Set'#13+
        #3'(C) 1990 by Dieter Schmalstieg',
        NIL, mfInformation+mfOkButton) = 0 THEN;
    ELSE
      Exit;
    END;
    ClearEvent(Event);
  END;
END;

PROCEDURE TMyApp.InitMenuBar;
VAR R: TRect;
BEGIN
  GetExtent(R);
  R.B.Y := R.A.Y + 1;
  MenuBar := New(PMenuBar, Init(R, NewMenu(
    NewSubMenu('~D~atei', hcn, NewMenu(
      NewItem('~L~aden', 'F3', kbF3, cmLaden, hcn,
      NewItem('~N~eu', '', kbNoKey, cmNeuDi, hcn,
      NewItem('~S~peichern', 'F2', kbF2,cmSpeic,hcn,
      NewItem('Speichern ~u~nter', '', kbNoKey, cmSpeiU,hcn,
      NewItem('S~o~urce schreiben', 'F4', kbF4, cmSourc,hcn,
      NewItem('~~ber DCS...', '', kbNoKey, cmUeber,hcn,
      NewLine(
      NewItem('~E~nde', 'Alt-X', kbAltX, cmQuit,hcn,
      NIL)))))))) ),
    NewSubMenu('Dialog', hcn, NewMenu(
      NewItem('~D~efinitionen', 'F8', kbF8, cmDefin,hcn,
      NewItem('Dialog ~t~esten', 'F9', kbF9, cmTeste,hcn,
      NIL)) ),
    NewSubMenu('~E~lement', hcn, NewMenu(
      NewItem('~N~eu',      'Alt-N', kbAltN, cmNeu,hcn,
      NewItem('~M~odifizieren','Alt-M', kbAltM, cmAende,hcn,
      NewItem('~K~opieren', 'Alt-K', kbAltK, cmKopie, hcn,
      NewItem('Kopieren & ~a~nhngen', 'Alt-A',
        kbAltA, cmKUA, hcn,
      NewItem('~L~schen',  'Alt-L', kbAltL, cmLoesc, hcn,
      NewItem('E~i~nfgen', 'Alt-I', kbAltI, cmEinfu, hcn,
      NIL))))))),
    NIL))
  ))));
END;

PROCEDURE TMyApp.InitStatusLine;
VAR R: TRect;
BEGIN
  GetExtent(R);
  R.A.Y := R.B.Y - 1;
  StatusLine := New(PStatusLine, Init(R,
    NewStatusDef(0, $FFFF,
      NewStatusKey('', kbF10, cmMenu,
      NewStatusKey('~Alt-X~ Ende', kbAltX, cmQuit,
      NIL)),
    NIL)
  ));
END;

PROCEDURE TMyApp.Definitionen;
VAR
  p: PView;
  Dialog: PDDialog;
  R: TRect;
  C: Word;
BEGIN
  R.Assign(10, 6, 70, 19);
  Dialog := New(PDDialog, Init(R, 'Definitionen'));
  with Dialog^ do
  BEGIN
    R.Assign(20,1,40,2);
    p := New(PInputline, Init(R, 20));
    Insert(p);
    R.Assign(1,1,19,2);
    Insert(New(PLabel, Init(R, '~~berschrift', p)));

    R.Assign(20,3,40,4);
    p := New(PInputline, Init(R, 20));
    Insert(p);
    R.Assign(1,3,19,4);
    Insert(New(PLabel, Init(R, '~P~rozedurname', p)));

    R.Assign(10,5,14,6);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(5,5,8,6);
    Insert(New(PLabel, Init(R, 'X1', p)));

    R.Assign(10,6,14,7);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(5,6,8,7);
    Insert(New(PLabel, Init(R, 'Y1', p)));

    R.Assign(30,5,34,6);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(25,5,28,6);
    Insert(New(PLabel, Init(R, 'X2', p)));

    R.Assign(30,6,34,7);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(25,6,28,7);
    Insert(New(PLabel, Init(R, 'Y2', p)));

    R.Assign(15, 10, 25, 12);
    Insert(New(PButton, Init(R, '~O~k', cmOK, bfDefault)));
    R.Assign(28, 10, 38, 12);
    Insert(New(PButton,
      Init(R,'Abbruch',cmCancel,bfNormal)));
    SetData(def);
  END;
  C := DeskTop^.ExecView(Dialog);
  IF C = cmOK THEN Dialog^.GetData(def);
  Dispose(Dialog, Done);
END;

FUNCTION TMyApp.Element;
VAR
  p : PView;
  Dialog: PDDialog;
  R : TRect;
  C : Word;
  i : INTEGER;
BEGIN
  R.Assign(5, 3, 75, 21);
  Dialog := New(PDDialog, Init(R, 'Element bearbeiten'));
  with Dialog^ do
  BEGIN
    R.Assign(1,2,20,10);
    p := New(PRadioButtons, Init(R,
      NewSItem('~B~utton',
      NewSItem('~I~nputLine',
      NewSItem('~C~heckBoxes',
      NewSItem('~R~adioButton',
      NewSItem('~S~taticText',
      NewSItem('~L~ongint',
      NewSItem('R~e~al',
      NewSItem('~D~atum',
      NIL))))))))
    ));
    Insert(p);
    R.Assign(1,1,20,2);
    Insert(New(PLabel, Init(R, 'Element~t~yp', p)));

    R.Assign(25,2,44,3);
    p := New(PCheckBoxes, Init(R,
      NewSItem('Label',
      NIL)
    ));
    Insert(p);

    R.Assign(25,3,44,4);
    p := New(PInputline, Init(R, 40));
    Insert(p);

    R.Assign(25,4,48,5);
    Insert(New(PStaticText, Init(R, 'Labelkoordinaten')));

    R.Assign(30,5,34,6);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(25,5,28,6);
    Insert(New(PLabel, Init(R, 'X1', p)));

    R.Assign(30,6,34,7);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(25,6,28,7);
    Insert(New(PLabel, Init(R, 'Y1', p)));

    R.Assign(40,5,44,6);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(35,5,38,6);
    Insert(New(PLabel, Init(R, 'X2', p)));

    R.Assign(40,6,44,7);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(35,6,38,7);
    Insert(New(PLabel, Init(R, 'Y2', p)));

    R.Assign(25,8,48,9);
    Insert(New(PStaticText, Init(R, 'Elementkoordinaten')));

    R.Assign(30,9,34,10);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(25,9,28,10);
    Insert(New(PLabel, Init(R, 'X1', p)));

    R.Assign(30,10,34,11);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(25,10,28,11);
    Insert(New(PLabel, Init(R, 'Y1', p)));

    R.Assign(40,9,44,10);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(35,9,38,10);
    Insert(New(PLabel, Init(R, 'X2', p)));

    R.Assign(40,10,44,11);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(35,10,38,11);
    Insert(New(PLabel, Init(R, 'Y2', p)));

    R.Assign(40,12,44,13);
    p := New(PLongint, Init(R, 2));
    Insert(p);
    R.Assign(24,12,38,13);
    Insert(New(PLabel, Init(R, 'Sub/Breite :', p)));

    FOR i := 1 TO 16 DO BEGIN
      R.Assign(46,i,68,i+1);
      Insert(New(PInputLine, Init(R, 20)));
    END;

    R.Assign( 1, 14, 11, 16);
    Insert(New(PButton, Init(R, '~O~k', cmOK, bfDefault)));
    R.Assign(15, 14, 28, 16);
    Insert(New(PButton,
      Init(R, 'Abbrechen', cmCancel, bfNormal)));

    Dialog^.SetData(e1);
  END;
  C := DeskTop^.ExecView(Dialog);
  Element := (C = cmOK);
  IF C = cmOK THEN Dialog^.GetData(e2);
  Dispose(Dialog, Done);
END;

VAR
  MyApp: TMyApp;

BEGIN
  New(e);
  ProgInit;
  MyApp.Init;
  MyApp.Run;
  MyApp.Done;
END.
