(* Name : TVPLUS.PAS               *)
(* Typ : Unit                      *)
(* Funktion : Dialogobjekte fr TV *)
(* Autor : Dieter Schmalstieg      *)
(* Projektstart : 91-01-27         *)
(* Letztes Update : 91-02-08       *)

UNIT TVplus;

INTERFACE

USES StrFunc, Objects, Views, Drivers, Dialogs, Dos;

TYPE
  TLongint = OBJECT (TInputLine)
    newsel : BOOLEAN;
    CONSTRUCTOR Init(VAR bounds : TRect; amaxlen : INTEGER);
    DESTRUCTOR Done; VIRTUAL;
    FUNCTION DataSize : WORD; VIRTUAL;
    PROCEDURE Draw; VIRTUAL;
    PROCEDURE GetData(VAR rec); VIRTUAL;
    PROCEDURE HandleEvent(VAR event : TEvent); VIRTUAL;
    PROCEDURE SetData(VAR rec); VIRTUAL;
    PROCEDURE Select; VIRTUAL;
  END;
  PLongint = ^TLongint;

  TReal = OBJECT (TInputLine)
    newsel : BOOLEAN;
    CONSTRUCTOR Init(VAR bounds : TRect; amaxlen : INTEGER);
    DESTRUCTOR Done; VIRTUAL;
    FUNCTION DataSize : WORD; VIRTUAL;
    PROCEDURE Draw; VIRTUAL;
    PROCEDURE GetData(VAR rec); VIRTUAL;
    PROCEDURE HandleEvent(VAR event : TEvent); VIRTUAL;
    PROCEDURE SetData(VAR rec); VIRTUAL;
    PROCEDURE Select; VIRTUAL;
  END;
  PReal = ^TReal;

  TDate = OBJECT (TInputLine)
    newsel : BOOLEAN;
    CONSTRUCTOR Init(VAR bounds : TRect);
    DESTRUCTOR Done; VIRTUAL;
    FUNCTION DataSize : WORD; VIRTUAL;
    PROCEDURE Draw; VIRTUAL;
    PROCEDURE GetData(VAR rec); VIRTUAL;
    PROCEDURE HandleEvent(VAR event : TEvent); VIRTUAL;
    PROCEDURE SetData(VAR rec); VIRTUAL;
    PROCEDURE Select; VIRTUAL;
  END;
  PDate = ^TDate;

  DateRec = ARRAY[1..3] OF WORD;
  PDateRec = ^DateRec;

IMPLEMENTATION

(* ------------------------------------------------------ *)
(*   TLONGINT                                             *)
(* ------------------------------------------------------ *)

  CONSTRUCTOR TLongint.Init(VAR bounds : TRect; amaxlen : INTEGER);
  BEGIN
    TInputLine.Init(bounds, amaxlen);
    Data^ := Copy('          ', 1, amaxlen-1)+'0';
    SelEnd := 0;
    newsel := TRUE;
  END;

  PROCEDURE TLongint.Select;
  BEGIN
    TInputLine.Select;
    newsel := TRUE;
  END;

  DESTRUCTOR TLongint.Done;
  BEGIN
    TInputLine.Done;
  END;

  FUNCTION TLongint.DataSize : WORD;
  BEGIN
    DataSize := SizeOf(LONGINT);
  END;

  PROCEDURE TLongint.Draw;
  VAR
    i : INTEGER;
  BEGIN
    IF (State AND sfSelected) = 0 THEN newsel := TRUE;
    IF newsel AND (Data^[maxlen] = ' ') THEN
      Data^[maxlen] := '0';
    SelEnd := 0;
    IF newsel THEN SelEnd := maxlen;
    CurPos := maxlen-1;
    TInputLine.Draw;
  END;

  PROCEDURE TLongint.GetData(VAR rec);
  VAR
    l : LONGINT;
  BEGIN
    l := Vall(Data^);
    Move(l, rec, SizeOf(l));
  END;

  PROCEDURE TLongint.HandleEvent(VAR event : TEvent);
  VAR
    i : INTEGER;
  BEGIN
    TView.HandleEvent(event);
    CASE event.What OF
      evKeyDown : BEGIN
      IF (event.CharCode IN ['0'..'9']) THEN BEGIN
        IF newsel THEN BEGIN
          newsel := FALSE;
          FOR i := 1 TO maxlen DO Data^[i] := ' ';
        END;
        IF Data^[1] = ' ' THEN BEGIN
          FOR i := 1 TO maxlen-1 DO Data^[i] := Data^[i+1];
          Data^[maxlen] := event.CharCode;
        END;
        Draw;
        ClearEvent(event);
      END;
      IF (event.KeyCode = kbBack) THEN BEGIN
        newsel := FALSE;
        FOR i := maxlen DOWNTO 2 DO Data^[i] := Data^[i-1];
        Data^[1] := ' ';
        Draw;
        ClearEvent(event);
      END;
      END; (* evkeydown *)
    END; (* case what *)
  END;

  PROCEDURE TLongint.SetData(VAR rec);
  BEGIN
    Data^ := Strl(LONGINT(rec), maxlen);
  END;

(* ------------------------------------------------------ *)
(*   TREAL                                                *)
(* ------------------------------------------------------ *)

  CONSTRUCTOR TReal.Init(VAR bounds : TRect; amaxlen : INTEGER);
  BEGIN
    TInputLine.Init(bounds, amaxlen);
    Data^ := Copy('          ', 1, amaxlen-4)+'0.00';
    newsel := TRUE;
  END;

  PROCEDURE TReal.Select;
  BEGIN
    TInputLine.Select;
    newsel := TRUE;
  END;

  DESTRUCTOR TReal.Done;
  BEGIN
    TInputLine.Done;
  END;

  FUNCTION TReal.DataSize : WORD;
  BEGIN
    DataSize := SizeOf(REAL);
  END;

  PROCEDURE TReal.Draw;
  VAR
    i : INTEGER;
  BEGIN
    IF (State AND sfSelected) = 0 THEN newsel := TRUE;
    FOR i := maxlen-3 TO maxlen DO
      IF newsel AND (Data^[i] = ' ') THEN
        Data^[i] := '0';
    SelEnd := 0;
    IF newsel THEN BEGIN
      SelEnd := maxlen;
      CurPos := maxlen-4;
    END;
    TInputLine.Draw;
  END;

  PROCEDURE TReal.GetData(VAR rec);
  VAR
    r : REAL;
  BEGIN
    r := Valr(Data^);
    Move(r, rec, SizeOf(r));
  END;

  PROCEDURE TReal.HandleEvent(VAR event : TEvent);
  VAR
    i : INTEGER;
  BEGIN
    TView.HandleEvent(event);
    CASE event.What OF
      evKeyDown : BEGIN
      IF (event.CharCode IN ['0'..'9']) THEN BEGIN
        IF CurPos = maxlen-4 THEN BEGIN
            IF newsel THEN BEGIN
              newsel := FALSE;
              FOR i := 1 TO maxlen-3 DO Data^[i] := ' ';
            END;
            IF Data^[1] = ' ' THEN BEGIN
              FOR i := 1 TO maxlen-4 DO Data^[i] := Data^[i+1];
              Data^[maxlen-3] := event.CharCode;
            END;
        END; (* CurPos = maxlen-4 *)
        IF (CurPos = maxlen-1) AND (Data^[maxlen] = ' ') THEN BEGIN
          Data^[maxlen] := event.CharCode;
        END; (* CurPos = maxlen-1 *)
        IF CurPos = maxlen-2 THEN BEGIN
          Data^[maxlen-1] := event.CharCode;
          CurPos := maxlen-1;
        END; (* CurPos = maxlen-2 *)
        Draw;
        ClearEvent(event);
      END; (* IF CharCode 0..9 *)
      IF (event.CharCode IN [',','.']) THEN BEGIN
        CurPos := maxlen-2;
        Data^[maxlen-1] := ' ';
        Data^[maxlen] := ' ';
        newsel := FALSE;
        Draw;
        ClearEvent(event);
      END; (* komma *)
      IF (event.KeyCode = kbBack) THEN BEGIN
        newsel := FALSE;
        IF CurPos = maxlen-4 THEN BEGIN
          FOR i := maxlen-3 DOWNTO 2 DO Data^[i] := Data^[i-1];
          Data^[1] := ' ';
        END;
        IF CurPos = maxlen-2 THEN BEGIN
          Data^[maxlen-1] := ' ';
          CurPos := maxlen-4;
        END;
        IF CurPos = maxlen-1 THEN BEGIN
          Data^[maxlen] := ' ';
          Data^[maxlen-1] := ' ';
          CurPos := maxlen-2;
        END;
        Draw;
        ClearEvent(event);
      END;
      END; (* evkeydown *)
    END; (* case what *)
  END;

  PROCEDURE TReal.SetData(VAR rec);
  BEGIN
    Data^ := Strr(REAL(rec), maxlen, 2);
  END;

(* ------------------------------------------------------ *)
(*   TDATE                                                *)
(* ------------------------------------------------------ *)

CONST
  ninty : STRING[8] = '90-01-01';

  CONSTRUCTOR TDate.Init(VAR bounds : TRect);
  VAR
    jj, mm, tt, wt, i : WORD;
  BEGIN
    TInputLine.Init(bounds, 8);
    GetDate(jj, mm, tt, wt);
    jj := jj MOD 100;
    ninty := Strl(jj,2)+'-'+Strl(mm,2)+'-'+Strl(tt,2);
    FOR i := 1 TO 8 DO IF ninty[i] = ' ' THEN ninty[i] := '0';
    Data^ := ninty;
    SelEnd := 0;
    newsel := TRUE;
  END;

  PROCEDURE TDate.Select;
  BEGIN
    TInputLine.Select;
    newsel := TRUE;
  END;

  DESTRUCTOR TDate.Done;
  BEGIN
    TInputLine.Done;
  END;

  FUNCTION TDate.DataSize : WORD;
  BEGIN
    DataSize := SizeOf(DateRec);
  END;

  PROCEDURE TDate.Draw;
  VAR
    i : INTEGER;
  BEGIN
    IF (State AND sfSelected) = 0 THEN newsel := TRUE;
    FOR i := 1 TO 8 DO
      IF newsel AND (Data^[i] = ' ') THEN Data^[i] := ninty[i];
    SelEnd := 0;
    IF newsel THEN SelEnd := maxlen;
    TInputLine.Draw;
  END;

  PROCEDURE TDate.GetData(VAR rec);
  VAR
    dp : PDateRec;
  BEGIN
    dp := @rec;
    Data^ := Strl(dp^[1],2)+'-'+Strl(dp^[2],2)+'-'+Strl(dp^[3],2);
  END;

  PROCEDURE TDate.HandleEvent(VAR event : TEvent);
  VAR
    i : INTEGER;
  BEGIN
    TView.HandleEvent(event);
    CASE event.What OF
      evKeyDown : BEGIN
      IF (event.CharCode IN ['0'..'9']) AND (CurPos < 8) THEN BEGIN
        IF newsel THEN BEGIN
          newsel := FALSE;
          Data^ := '  -  -  ';
          CurPos := 0;
        END;
        Data^[CurPos+1] := event.CharCode;
        IF CurPos < 7 THEN Inc(CurPos);
        IF CurPos IN [2,5] THEN Inc(CurPos);
        Draw;
        ClearEvent(event);
      END;
      IF (event.KeyCode = kbBack) AND NOT newsel THEN BEGIN
        Data^[CurPos+1] := ' ';
        IF CurPos > 0 THEN Dec(CurPos);
        IF CurPos IN [2,5] THEN Dec(CurPos);
        Draw;
        ClearEvent(event);
      END;
      END; (* evkeydown *)
    END; (* case what *)
  END;

  PROCEDURE TDate.SetData(VAR rec);
  VAR
    dp : PDateRec;
  BEGIN
    dp := @rec;
    dp^[1] := WORD(Vall(Copy(Data^,1,2)));
    dp^[2] := WORD(Vall(Copy(Data^,4,2)));
    dp^[3] := WORD(Vall(Copy(Data^,7,2)));
  END;

END. (* TVplus *)
